from datetime import date
from typing import Any, Dict, List, Optional, Union

import attr

import infima_client.api.cohort_service.cohort_service_get_cohort_associations as cohort_service_get_cohort_associations
import infima_client.api.cohort_service.cohort_service_get_cohort_summary as cohort_service_get_cohort_summary
import infima_client.api.cohort_service.cohort_service_get_cohorts_associations as cohort_service_get_cohorts_associations
import infima_client.api.cohort_service.cohort_service_get_coverage_summary as cohort_service_get_coverage_summary
import infima_client.api.cohort_service.cohort_service_get_historical_prepayment as cohort_service_get_historical_prepayment
import infima_client.api.cohort_service.cohort_service_get_historical_prepayments as cohort_service_get_historical_prepayments
import infima_client.api.cohort_service.cohort_service_get_pool_associations as cohort_service_get_pool_associations
import infima_client.api.cohort_service.cohort_service_get_pool_components as cohort_service_get_pool_components
import infima_client.api.cohort_service.cohort_service_get_pools_associations as cohort_service_get_pools_associations
import infima_client.api.cohort_service.cohort_service_search_cohort as cohort_service_search_cohort
from infima_client.models import *

from .client import Client
from .types import UNSET, Response, Unset
from .util import named_arguments_only


class cohort:
    def __init__(self, client: Client):
        self.client = client
        self._detailed = cohortDetailed(client)

    @named_arguments_only
    def get_cohort_associations(
        self,
        cohort: str,
        as_of: Optional[date] = None,
        month_back: Optional[int] = 3,
    ) -> Optional[
        Union[
            CohortAssociations,
            CohortAssociationsUnauthorized,
            CohortAssociationsNotFound,
            CohortAssociationsError,
        ]
    ]:

        """
        Fetch the cusips associated to a given cohort.


        Args:
            cohort: Cohort for which to get associations. [Required]
            as_of: Security factor date from which to fetch the associations. [Optional: default to current factor date]
            month_back: Number of months previous for which to look for associations. [Optional: default to previous 3 months]

        Returns:
            Either a valid response or an unauthorized, not found or other error.
        """

        if as_of:
            year, month, day = as_of.year, as_of.month, as_of.day
        else:
            # This is the default for the method call
            year, month, day = 0, 0, 0

        return cohort_service_get_cohort_associations.sync(
            client=self.client,
            cohort=cohort,
            dateyear=year,
            datemonth=month,
            dateday=day,
            month_back=month_back,
        )

    @named_arguments_only
    def get_cohorts_associations(
        self,
        cohorts: List[str],
        as_of: Optional[date] = None,
        month_back: Optional[int] = 3,
    ) -> Optional[
        Union[
            CohortAssociations,
            CohortAssociationsUnauthorized,
            CohortAssociationsNotFound,
            CohortAssociationsError,
        ]
    ]:

        """
        Fetch the cusips associated to a given cohort.


        Args:
            cohorts: Cohorts for which to get associations. [Required]
            as_of: Security factor date from which to fetch the associations. [Optional: default to current factor date]
            month_back: Number of months previous for which to look for associations. [Optional: default to previous 3 months]

        Returns:
            Either a valid response or an unauthorized, not found or other error.
        """

        if as_of:
            year, month, day = as_of.year, as_of.month, as_of.day
        else:
            # This is the default for the method call
            year, month, day = 0, 0, 0

        return cohort_service_get_cohorts_associations.sync(
            client=self.client,
            cohorts=cohorts,
            dateyear=year,
            datemonth=month,
            dateday=day,
            month_back=month_back,
        )

    @named_arguments_only
    def get_cohort_summary(
        self,
        cohort: str,
        as_of: Optional[date] = None,
    ) -> Optional[
        Union[
            CohortSummary,
            CohortSummaryUnauthorized,
            CohortSummaryNotFound,
            CohortSummaryError,
        ]
    ]:

        """
        Fetch coverage summary for a cohort.


        Args:
            cohort: Cohort for which to get associations. [Required]
            as_of: Security factor date from which to fetch the associations. [Optional: default to current factor date]

        Returns:
            Either a valid response or an unauthorized, not found or other error.
        """

        if as_of:
            year, month, day = as_of.year, as_of.month, as_of.day
        else:
            # This is the default for the method call
            year, month, day = 0, 0, 0
        return cohort_service_get_cohort_summary.sync(
            client=self.client,
            cohort=cohort,
            dateyear=year,
            datemonth=month,
            dateday=day,
        )

    @named_arguments_only
    def get_coverage_summary(
        self,
        as_of: Optional[date] = None,
        month_back: Optional[int] = 3,
    ) -> Optional[
        Union[
            CohortCoverage,
            CohortCoverageUnauthorized,
            CohortCoverageNotFound,
            CohortCoverageError,
        ]
    ]:

        """
        This is a summary of infima coverage.


        Args:
            as_of: Security factor date from which to fetch the associations. [Optional: default to current factor date]
            month_back: Number of months previous for which to look for associations. [Optional: default to previous 3 months]

        Returns:
            Either a valid response or an unauthorized, not found or other error.
        """
        if as_of:
            year, month, day = as_of.year, as_of.month, as_of.day
        else:
            # This is the default for the method call
            year, month, day = 0, 0, 0
        return cohort_service_get_coverage_summary.sync(
            client=self.client,
            dateyear=year,
            datemonth=month,
            dateday=day,
            month_back=month_back,
        )

    @named_arguments_only
    def get_historical_prepayment(
        self,
        cohort: Union[Unset, str] = UNSET,
        periodyear: Union[Unset, int] = UNSET,
        periodmonth: Union[Unset, int] = UNSET,
        convention: Union[
            Unset, CohortService_GetHistoricalPrepaymentConvention
        ] = CohortService_GetHistoricalPrepaymentConvention.PERIOD,
        num_months: Union[Unset, int] = UNSET,
    ) -> Optional[
        Union[
            CohortHistoricalPrepayment,
            CohortHistoricalPrepaymentUnauthorized,
            CohortHistoricalPrepaymentNotFound,
            CohortHistoricalPrepaymentError,
        ]
    ]:

        """


        Args:
            cohort: Cohort to fetch.
            periodyear: Year of the period. Must be from 1 to 9999, or 0 to specify a date without a year
            periodmonth: Month of a year. Must be from 1 to 12, or 0 to specify a year without a month and day.
            convention: Convention to use for the date, can be PERIOD or FACTOR_DATE.
            num_months: None

        Returns:
            Either a valid response or an unauthorized, not found or other error.
        """

        return cohort_service_get_historical_prepayment.sync(
            client=self.client,
            cohort=cohort,
            periodyear=periodyear,
            periodmonth=periodmonth,
            convention=convention,
            num_months=num_months,
        )

    @named_arguments_only
    def get_historical_prepayments(
        self,
        cohorts: Union[Unset, List[str]] = UNSET,
        convention: Union[
            Unset, CohortService_GetHistoricalPrepaymentsJsonBodyConvention
        ] = CohortService_GetHistoricalPrepaymentsJsonBodyConvention.PERIOD,
        num_months: Union[Unset, int] = UNSET,
        period: Union[
            CohortService_GetHistoricalPrepaymentsJsonBodyPeriod, Unset
        ] = UNSET,
    ) -> Optional[
        Union[
            CohortHistoricalPrepayments,
            CohortHistoricalPrepaymentsUnauthorized,
            CohortHistoricalPrepaymentsNotFound,
            CohortHistoricalPrepaymentsError,
        ]
    ]:

        """


        Args:

        Returns:
            Either a valid response or an unauthorized, not found or other error.
        """

        return cohort_service_get_historical_prepayments.sync(
            client=self.client,
            json_body=CohortService_GetHistoricalPrepaymentsJsonBody(
                cohorts=cohorts,
                convention=convention,
                num_months=num_months,
                period=period,
            ),
        )

    @named_arguments_only
    def get_pool_associations(
        self,
        cusip: str,
        as_of: Optional[date] = None,
        month_back: Optional[int] = 3,
    ) -> Optional[
        Union[
            PoolAssociations,
            PoolAssociationsUnauthorized,
            PoolAssociationsNotFound,
            PoolAssociationsError,
        ]
    ]:

        """
        Get the cohorts associated to a CUSIP.

        Args:
            cusip: CUSIP for which we want to find the associated cohorts.
            as_of: Security factor date from which to fetch the associations. [Optional: default to current factor date]
            month_back: Number of months previous for which to look for associations. [Optional: default to previous 3 months]

        Returns:
            Either a valid response or an unauthorized, not found or other error.
        """

        if as_of:
            year, month, day = as_of.year, as_of.month, as_of.day
        else:
            # This is the default for the method call
            year, month, day = 0, 0, 0
        return cohort_service_get_pool_associations.sync(
            client=self.client,
            cusip=cusip,
            dateyear=year,
            datemonth=month,
            dateday=day,
            month_back=month_back,
        )

    @named_arguments_only
    def get_pool_components(
        self, cohort: str
    ) -> Optional[
        Union[
            CohortPoolComponents,
            CohortPoolComponentsUnauthorized,
            CohortPoolComponentsNotFound,
            CohortPoolComponentsError,
        ]
    ]:

        """

        Get the pool components for a cohort.

        Args:
            cohort: Cohort we want to fetch pool components for.

        Returns:
            Either a valid response or an unauthorized, not found or other error.
        """

        return cohort_service_get_pool_components.sync(
            client=self.client, cohort=cohort
        )

    @named_arguments_only
    def get_pools_associations(
        self,
        cusips: List[str],
        as_of: Optional[date] = None,
        month_back: Optional[int] = 3,
    ) -> Optional[
        Union[
            PoolsAssociations,
            PoolsAssociationsUnauthorized,
            PoolsAssociationsNotFound,
            PoolsAssociationsError,
        ]
    ]:

        """
        Get the cohorts associated to a list of CUSIPs


        Args:
            cusips: CUSIPs for which we want to find the associated cohorts.
            as_of: Security factor date from which to fetch the associations. [Optional: default to current factor date]
            month_back: Number of months previous for which to look for associations. [Optional: default to previous 3 months]

        Returns:
            Either a valid response or an unauthorized, not found or other error.
        """

        if as_of:
            year, month, day = as_of.year, as_of.month, as_of.day
        else:
            # This is the default for the method call
            year, month, day = 0, 0, 0
        return cohort_service_get_pools_associations.sync(
            client=self.client,
            cusips=cusips,
            dateyear=year,
            datemonth=month,
            dateday=day,
            month_back=month_back,
        )

    @named_arguments_only
    def search_cohort(
        self,
        ticker: Optional[str] = None,
        coupon: Optional[float] = None,
        vintage: Optional[int] = None,
        type: Optional[str] = None,
    ) -> Optional[
        Union[
            CohortSearch,
            CohortSearchUnauthorized,
            CohortSearchNotFound,
            CohortSearchError,
        ]
    ]:

        """


        Args:
            ticker: None
            coupon: None
            vintage: None
            type: None

        Returns:
            Either a valid response or an unauthorized, not found or other error.
        """

        return cohort_service_search_cohort.sync(
            client=self.client, ticker=ticker, coupon=coupon, vintage=vintage, type=type
        )


class cohortDetailed:
    def __init__(self, client: Client):
        self.client = client

    @named_arguments_only
    def get_cohort_associations(
        self,
        cohort: str,
        as_of: Optional[date] = None,
        month_back: Optional[int] = 3,
    ) -> Response[
        Union[
            CohortAssociations,
            CohortAssociationsUnauthorized,
            CohortAssociationsNotFound,
            CohortAssociationsError,
        ]
    ]:

        if as_of:
            year, month, day = as_of.year, as_of.month, as_of.day
        else:
            # This is the default for the method call
            year, month, day = 0, 0, 0

        return cohort_service_get_cohort_associations.sync_detailed(
            client=self.client,
            cohort=cohort,
            dateyear=year,
            datemonth=month,
            dateday=day,
            month_back=month_back,
        )

    @named_arguments_only
    def get_cohorts_associations(
        self,
        cohorts: List[str],
        as_of: Optional[date] = None,
        month_back: Optional[int] = 3,
    ) -> Optional[
        Union[
            CohortAssociations,
            CohortAssociationsUnauthorized,
            CohortAssociationsNotFound,
            CohortAssociationsError,
        ]
    ]:

        """
        Fetch the cusips associated to a given cohort.


        Args:
            cohorts: Cohorts for which to get associations. [Required]
            as_of: Security factor date from which to fetch the associations. [Optional: default to current factor date]
            month_back: Number of months previous for which to look for associations. [Optional: default to previous 3 months]

        Returns:
            Either a valid response or an unauthorized, not found or other error.
        """

        if as_of:
            year, month, day = as_of.year, as_of.month, as_of.day
        else:
            # This is the default for the method call
            year, month, day = 0, 0, 0

        return cohort_service_get_cohorts_associations.sync_detailed(
            client=self.client,
            cohorts=cohorts,
            dateyear=year,
            datemonth=month,
            dateday=day,
            month_back=month_back,
        )

    @named_arguments_only
    def get_cohort_summary(
        self,
        cohort: str,
        as_of: Optional[date] = None,
    ) -> Response[
        Union[
            CohortSummary,
            CohortSummaryUnauthorized,
            CohortSummaryNotFound,
            CohortSummaryError,
        ]
    ]:
        if as_of:
            year, month, day = as_of.year, as_of.month, as_of.day
        else:
            # This is the default for the method call
            year, month, day = 0, 0, 0

        return cohort_service_get_cohort_summary.sync_detailed(
            client=self.client,
            cohort=cohort,
            dateyear=year,
            datemonth=month,
            dateday=day,
        )

    @named_arguments_only
    def get_coverage_summary(
        self,
        as_of: Optional[date] = None,
        month_back: Optional[int] = 3,
    ) -> Response[
        Union[
            CohortCoverage,
            CohortCoverageUnauthorized,
            CohortCoverageNotFound,
            CohortCoverageError,
        ]
    ]:
        if as_of:
            year, month, day = as_of.year, as_of.month, as_of.day
        else:
            # This is the default for the method call
            year, month, day = 0, 0, 0
        return cohort_service_get_coverage_summary.sync_detailed(
            client=self.client,
            dateyear=year,
            datemonth=month,
            dateday=day,
            month_back=month_back,
        )

    @named_arguments_only
    def get_historical_prepayment(
        self,
        cohort: Union[Unset, str] = UNSET,
        periodyear: Union[Unset, int] = UNSET,
        periodmonth: Union[Unset, int] = UNSET,
        convention: Union[
            Unset, CohortService_GetHistoricalPrepaymentConvention
        ] = CohortService_GetHistoricalPrepaymentConvention.PERIOD,
        num_months: Union[Unset, int] = UNSET,
    ) -> Response[
        Union[
            CohortHistoricalPrepayment,
            CohortHistoricalPrepaymentUnauthorized,
            CohortHistoricalPrepaymentNotFound,
            CohortHistoricalPrepaymentError,
        ]
    ]:

        return cohort_service_get_historical_prepayment.sync_detailed(
            client=self.client,
            cohort=cohort,
            periodyear=periodyear,
            periodmonth=periodmonth,
            convention=convention,
            num_months=num_months,
        )

    @named_arguments_only
    def get_historical_prepayments(
        self,
        cohorts: Union[Unset, List[str]] = UNSET,
        convention: Union[
            Unset, CohortService_GetHistoricalPrepaymentsJsonBodyConvention
        ] = CohortService_GetHistoricalPrepaymentsJsonBodyConvention.PERIOD,
        num_months: Union[Unset, int] = UNSET,
        period: Union[
            CohortService_GetHistoricalPrepaymentsJsonBodyPeriod, Unset
        ] = UNSET,
    ) -> Response[
        Union[
            CohortHistoricalPrepayments,
            CohortHistoricalPrepaymentsUnauthorized,
            CohortHistoricalPrepaymentsNotFound,
            CohortHistoricalPrepaymentsError,
        ]
    ]:

        return cohort_service_get_historical_prepayments.sync_detailed(
            client=self.client,
            json_body=GetPrepaymentsRequest(
                cohorts=cohorts,
                convention=convention,
                num_months=num_months,
                period=period,
            ),
        )

    @named_arguments_only
    def get_pool_associations(
        self,
        cusip: str,
        as_of: Optional[date] = None,
        month_back: Optional[int] = 3,
    ) -> Response[
        Union[
            PoolAssociations,
            PoolAssociationsUnauthorized,
            PoolAssociationsNotFound,
            PoolAssociationsError,
        ]
    ]:

        if as_of:
            year, month, day = as_of.year, as_of.month, as_of.day
        else:
            # This is the default for the method call
            year, month, day = 0, 0, 0
        return cohort_service_get_pool_associations.sync_detailed(
            client=self.client,
            cusip=cusip,
            dateyear=year,
            datemonth=month,
            dateday=day,
            month_back=month_back,
        )

    @named_arguments_only
    def get_pool_components(
        self, cohort: str
    ) -> Response[
        Union[
            CohortPoolComponents,
            CohortPoolComponentsUnauthorized,
            CohortPoolComponentsNotFound,
            CohortPoolComponentsError,
        ]
    ]:

        return cohort_service_get_pool_components.sync_detailed(
            client=self.client, cohort=cohort
        )

    @named_arguments_only
    def get_pools_associations(
        self,
        cusips: List[str],
        as_of: Optional[date] = None,
        month_back: Optional[int] = 3,
    ) -> Response[
        Union[
            PoolsAssociations,
            PoolsAssociationsUnauthorized,
            PoolsAssociationsNotFound,
            PoolsAssociationsError,
        ]
    ]:

        if as_of:
            year, month, day = as_of.year, as_of.month, as_of.day
        else:
            # This is the default for the method call
            year, month, day = 0, 0, 0
        return cohort_service_get_pools_associations.sync_detailed(
            client=self.client,
            cusips=cusips,
            dateyear=year,
            datemonth=month,
            dateday=day,
            month_back=month_back,
        )

    @named_arguments_only
    def search_cohort(
        self,
        ticker: Optional[str] = None,
        coupon: Optional[float] = None,
        vintage: Optional[int] = None,
        type: Optional[str] = None,
    ) -> Response[
        Union[
            CohortSearch,
            CohortSearchUnauthorized,
            CohortSearchNotFound,
            CohortSearchError,
        ]
    ]:

        return cohort_service_search_cohort.sync_detailed(
            client=self.client, ticker=ticker, coupon=coupon, vintage=vintage, type=type
        )
